import JaCoP.core.IntVar;
import JaCoP.search.DepthFirstSearch;
import JaCoP.search.IndomainMin;
import JaCoP.search.Search;
import JaCoP.search.SelectChoicePoint;
import JaCoP.search.SimpleSelect;
import JaCoP.search.SmallestDomain;
import SoftLib.constraints.SoftExtensionalSupportSTR;
import SoftLib.constraints.SoftXltC;
import SoftLib.core.IntCouple;
import SoftLib.core.SoftStore;
import SoftLib.weight.WeightExtensional;
import SoftLib.weight.WeightUnaryExtensional;
import java.util.HashMap;


public class Esempio_softArcConsistency_report {
    public static void main(String[] args) {

        SoftStore softStore = new SoftStore(SoftStore.Semiring.FUZZY);
        
        IntVar x = new IntVar(softStore, "X", 1, 3);
        SoftXltC cx = new SoftXltC(x, 4);
        cx.impose(softStore, new WeightUnaryExtensional(new int[] {700, 500, 300}));
        
        IntVar y = new IntVar(softStore, "Y", 1, 3);
        SoftXltC cy = new SoftXltC(y, 4);
        cy.impose(softStore, new WeightUnaryExtensional(new int[] {400, 200, 600}));
        
        IntVar[] list = {x, y};
        int[][] tuples;
        tuples = new int[][]{{1, 1}, {1, 2}, {1, 3},
                             {2, 1}, {2, 2}, {2, 3},
                             {3, 1}, {3, 2}, {3, 3}};
        SoftExtensionalSupportSTR cxy = new SoftExtensionalSupportSTR(list, tuples);
        cxy.impose(softStore, new WeightExtensional(new int[]{700, 0, 300, 0, 0, 500, 400, 0, 800}));
        
        softStore.softArcConsistency();
        
        System.out.println("NEW PREFERENCE VALUE");
        System.out.println("Cx:");
        HashMap<Integer, Integer> xWeights = softStore.getwStore().wUnary.get(x).w.get(0);
        for(int i : x.domain.toIntArray()){
            System.out.println(" "+i+" -> "+xWeights.get(i));
        }

        System.out.println("Cy:");
        HashMap<Integer, Integer> yWeights = softStore.getwStore().wUnary.get(y).w.get(0);
        for(int i : y.domain.toIntArray()){
            System.out.println(" "+i+" -> "+yWeights.get(i));
        }

        System.out.println("Cxy:");
        System.out.println("      +-----+-----+-----+");
        System.out.println("  X\\Y |  1  |  2  |  3  |");
        System.out.println("+-----+-----+-----+-----+");
        HashMap<IntCouple, Integer> xyWeight = softStore.getwStore().wBinary.get(0).w.get(0);
        for(int i : x.domain.toIntArray()) {
            System.out.print("|  "+ i +"  |");
            for(int j : y.domain.toIntArray()){
                System.out.print(" "+String.format("%3d",xyWeight.get(new IntCouple(i, j)))+" |");
            }
        System.out.println("\n+-----+-----+-----+-----+");
        }
        System.out.println();
        
        softStore.imposeWeights();
        
        Search<IntVar> label = new DepthFirstSearch<IntVar>();
        SelectChoicePoint<IntVar> select = new SimpleSelect<IntVar>(list,
                new SmallestDomain<IntVar>(),
                new IndomainMin<IntVar>());

        label.getSolutionListener().searchAll(true);
        label.getSolutionListener().recordSolutions(true);

        label.labeling(softStore, select, softStore.getCost());

        label.printAllSolutions();
        
    }
}

