package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.Max;
import JaCoP.core.IntVar;
import SoftLib.core.SoftStore;

/** 
 * This class imposes a cost using by the weight of the maximum.
 */

public class WeightMax extends PrimitiveBinaryWeight {

    public WeightMax() {
    }

    /*
        this method sets the biggest value as the cost of the constraint
     */
    @Override
    public Constraint setWeight(SoftStore store, IntVar x, IntVar y) {
        
        int min = Math.max(x.min(), y.min());
        int max = Math.max(x.max(), y.max());
        if((store.getSrType() == SoftStore.Semiring.FUZZY ||
            store.getSrType() == SoftStore.Semiring.PROBABILISTIC) &&
            (max > 1000 || min < 0)) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == SoftStore.Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be less than 0 or greater than 1000");
        }

        IntVar z = new IntVar(store, "store.cost", min , max);
        store.costs.add(z);
        
        IntVar[] v = {x, y};
        Constraint cost = new Max(v, z);
        return cost;
    }

    @Override
    public int evaluateWeight(int x, int y) {
        return Math.max(x, y);
    }
}